package com.devdaily.snapshot;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

public class Snapshot
{
  SnapshotMainFrame mainFrame;
  SnapshotTransparentFrame shield;
  final Snapshot thisController;
  private BufferedImage currentImage;
  JFileChooser fc = null;
  private boolean isMac;

  public static void main(String[] args)
  {
    new Snapshot();
  }

  public Snapshot()
  {
    thisController = this;

    // kludge
    boolean mrjVersionExists = System.getProperty("mrj.version") != null;
    boolean osNameExists = System.getProperty("os.name").startsWith("Mac OS");
    if ( !mrjVersionExists || !osNameExists)
    {
      isMac = false;
    }
    else
    {
      isMac = true;
    }

    mainFrame = new SnapshotMainFrame(this);
    mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    mainFrame.setLocationRelativeTo(null);
    mainFrame.setVisible(true);
  }

  public void saveImageAsFileAction() {
    // instantiate this just once and it will remember your last dir
    if (fc == null) {
      fc = new JFileChooser();
    }
//  ExampleFileFilter filter = new ExampleFileFilter();
//  filter.addExtension("png");
//  filter.addExtension("gif");
//  filter.setDescription("JPG & GIF Images");
//  fc.setFileFilter(filter);
    int returnVal = fc.showSaveDialog(mainFrame);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      try {
        ImageIO.write(currentImage, "png", f);
      } catch (IOException e) {
        e.printStackTrace();
        JOptionPane.showMessageDialog(new JLabel("An error occurred"), null);
      }
    }
  }
  
 
  /**
   * This method should be called by the transparent frame so we will save the image.
   */
  public void doSaveImageAction(int x, int y, int h, int w, BufferedImage image)
  {
    disposeShield();
    
    // re-display main frame, then prompt to save the file
    mainFrame.setSnapshot(image);
    mainFrame.validate();
    mainFrame.setVisible(true);
    
    currentImage = image;
  }

  private void disposeShield() {
    shield.setVisible(false);
    shield.dispose();
    shield = null;
  }

  public void doExitAction()
  {
    System.exit(0);
  }

  public void doDisplayShieldAction()
  {
    // hide the main frame
    mainFrame.setVisible(false);

    // sleep to let the user do a little setup
    try { Thread.sleep(500);  }  catch (InterruptedException ie) {}

    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        shield = new SnapshotTransparentFrame(thisController, isMac);
        shield.setVisible(true);
        shield.repaint();
      }
    });
    
  }
}

// next two classes from sun

class ImageFilter extends FileFilter {

  //Accept all directories and all gif, jpg, tiff, or png files.
  public boolean accept(File f) {
      if (f.isDirectory()) {
          return true;
      }

      String extension = Utils.getExtension(f);
      if (extension != null) {
          if (extension.equals(Utils.tiff) ||
              extension.equals(Utils.tif) ||
              extension.equals(Utils.gif) ||
              extension.equals(Utils.jpeg) ||
              extension.equals(Utils.jpg) ||
              extension.equals(Utils.png)) {
                  return true;
          } else {
              return false;
          }
      }

      return false;
  }

  //The description of this filter
  public String getDescription() {
      return "Just Images";
  }
}

class Utils {
  public final static String jpeg = "jpeg";
  public final static String jpg = "jpg";
  public final static String gif = "gif";
  public final static String tiff = "tiff";
  public final static String tif = "tif";
  public final static String png = "png";

  /*
   * Get the extension of a file.
   */
  public static String getExtension(File f) {
      String ext = null;
      String s = f.getName();
      int i = s.lastIndexOf('.');

      if (i > 0 &&  i < s.length() - 1) {
          ext = s.substring(i+1).toLowerCase();
      }
      return ext;
  }

  /** Returns an ImageIcon, or null if the path was invalid. */
  protected static ImageIcon createImageIcon(String path) {
      java.net.URL imgURL = Utils.class.getResource(path);
      if (imgURL != null) {
          return new ImageIcon(imgURL);
      } else {
          System.err.println("Couldn't find file: " + path);
          return null;
      }
  }
}

