package com.devdaily.snapshot;

import java.awt.Image;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import javax.swing.*;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

public class SnapshotMainFrame extends JFrame
{
  private JButton goButton = new JButton("Take Snapshot");
  private JButton quitButton = new JButton("Quit");
  private JButton saveButton = new JButton("Save Image");
  Snapshot controller;
  
  // to display the current image
  Image currentImage;
  JLabel imageLabel;

  public SnapshotMainFrame(Snapshot controller)
  {
    this.controller = controller;
    setTitle("Snapshot Creator");
    goButton.addActionListener(new XYMainFrame_goButton_actionAdapter(this));
    quitButton.addActionListener(new XYMainFrame_quitButton_actionAdapter(this));
    saveButton.addActionListener(new XYMainFrame_saveButton_actionAdapter(this));
    saveButton.setEnabled(false);
    getContentPane().add(buildPanel(), java.awt.BorderLayout.CENTER);
    pack();
  }
  
  public void setSnapshot(Image image) {
    currentImage = image;
    saveButton.setEnabled(true);
    imageLabel.setText("");
    imageLabel.setIcon(new ImageIcon(currentImage));
    getContentPane().validate();
    this.pack();
    this.setLocationRelativeTo(null);
  }

  private JComponent buildPanel() {

    //@todo Fix this layout
    FormLayout layout = new FormLayout("pref:grow",
                                       "pref, 6dlu,pref,3dlu,pref,3dlu,pref:grow,3dlu,pref");
    DefaultFormBuilder builder = new DefaultFormBuilder(layout);
    builder.setDefaultDialogBorder();
    CellConstraints cc = new CellConstraints();

    JLabel header = new JLabel("Snapshot Creator");
    builder.add(header, cc.xy(1, 1));

    JLabel helpText1 = new JLabel("After starting the process select two points on the screen");
    builder.add(helpText1, cc.xy(1, 3));

    JLabel helpText2 = new JLabel("to select your rectangle, then save your file.");
    builder.add(helpText2, cc.xy(1, 5));

    if (currentImage==null) {
      imageLabel = new JLabel("<no image to display>");
    } else {
      imageLabel = new JLabel(new ImageIcon(currentImage));
    }
    builder.add(imageLabel, cc.xy(1, 7));

    builder.add(ButtonBarFactory.buildCenteredBar(goButton, saveButton, quitButton), cc.xy(1, 9));
    return builder.getPanel();
  }

  public void goButton_actionPerformed(ActionEvent e)
  {
    controller.doDisplayShieldAction();
  }

  public void quitButton_actionPerformed(ActionEvent e)
  {
    controller.doExitAction();
  }

  public void saveButton_actionPerformed(ActionEvent e)
  {
    controller.saveImageAsFileAction();
  }
}

class XYMainFrame_quitButton_actionAdapter implements ActionListener
{
  private SnapshotMainFrame adaptee;
  XYMainFrame_quitButton_actionAdapter(SnapshotMainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.quitButton_actionPerformed(e);
  }
}

class XYMainFrame_saveButton_actionAdapter implements ActionListener
{
  private SnapshotMainFrame adaptee;
  XYMainFrame_saveButton_actionAdapter(SnapshotMainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.saveButton_actionPerformed(e);
  }
}

class XYMainFrame_goButton_actionAdapter implements ActionListener
{
  private SnapshotMainFrame adaptee;
  XYMainFrame_goButton_actionAdapter(SnapshotMainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.goButton_actionPerformed(e);
  }
}

