package com.devdaily.snapshot;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import javax.imageio.ImageIO;
import javax.swing.*;

public class SnapshotTransparentFrame extends JFrame {
  protected BufferedImage background;
  private JLabel jlabel = new JLabel();
  private int yOffset = 0;
  private JPanel contentPane;
  private int x1, y1, x2, y2;
  private BufferedImage snapshot;
  private int clickCount = 0;
  Snapshot controller;
  
  // trying to make a fancy cursor
  Graphics2D g2d;
  BufferedImage mouseCursorImage;
  Cursor myCursor;
  Toolkit toolkit;
  Point cursorPoint;
  
  private boolean isMac;

  public SnapshotTransparentFrame(Snapshot controller, boolean isMac) 
  {
    super();
    this.controller = controller;
    this.isMac = isMac;
    contentPane = (JPanel) this.getContentPane();
    contentPane.setDoubleBuffered(true);
    getContentPane().setLayout(new BorderLayout());
    // frame behavior
    setAlwaysOnTop(true);
    setUndecorated(true);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setSize(screenSize.width, screenSize.height - yOffset);
    addMouseListener();
    // magically set the image on the jlabel :)
    getBackgroundImage();
    getContentPane().add("North", jlabel);
    
    class AlsMouseLabel extends JComponent
    {
      public int x;
      public int y;
      
      public AlsMouseLabel() {
        this.setBackground(Color.blue);
      }
      
      protected void paintComponent(Graphics g)
      {
        super.paintComponent(g);
        String s = x + ", " + y;
        g.setColor(Color.red);
        g.drawString(s, x, y+20);
      }
    }

    final AlsMouseLabel mouseLabel = new AlsMouseLabel();
    //this.setGlassPane(mouseLabel);
    //this.getGlassPane().setVisible(true);

    JLayeredPane layeredPane = getRootPane().getLayeredPane();
    layeredPane.add(mouseLabel, JLayeredPane.DRAG_LAYER);
    mouseLabel.setBounds(0, 0, getWidth(), getHeight());
    
    this.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent me)
      {
        int x = me.getX();
        int y = me.getY();
        mouseLabel.x = x;
        mouseLabel.y = y;
        mouseLabel.repaint();
      }
    });
    
    // ORIGINAL
    setCursor(Cursor.CROSSHAIR_CURSOR);
  }

  void addMouseListener() {
    this.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        int button = e.getButton();
        if (button == MouseEvent.BUTTON1) {
          if (clickCount == 0) {
            x1 = e.getX();
            y1 = e.getY();
            clickCount++;
          } else {
            x2 = e.getX();
            y2 = e.getY();
            // get the sub image
            snapshot = background.getSubimage(getX(), getY(), getW(), getH());
            controller.doSaveImageAction(getX(), getY(), getH(), getW(), snapshot);
          }
        }
      }
    });
  }

  public void paint(Graphics g) {
    super.paint(g);
    g.setColor(Color.ORANGE);
    g.drawString("OVERLAY", 10, 20);
  }

  public void getBackgroundImage() {
    try {
      Robot rbt = new Robot();
      Toolkit tk = Toolkit.getDefaultToolkit();
      Dimension dim = tk.getScreenSize();
      if (isMac)
      {
        yOffset = 22;
      }
      background = rbt.createScreenCapture(new Rectangle(0, yOffset, dim.width, dim.height-yOffset));

      // old
      //background = rbt.createScreenCapture(new Rectangle(0, yOffset - 1, (int) dim.getWidth(), (int) dim.getHeight()));
      jlabel.setIcon(new ImageIcon(background));
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public int getH() {
    return y2 - y1;
  }

  public BufferedImage getSnapshot() {
    return snapshot;
  }

  public int getW() {
    return x2 - x1;
  }

  public int getX() {
    return x1;
  }

  public int getY() {
    return y1;
  }

}
