package com.devdaily.xylocator;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;

/**
 * Copyright 2010, Alvin J. Alexander, devdaily.com.
 * 
 * This file is part of the devdaily.com XYLocator application.
 * 
 * XYLocator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * XYLocator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XYLocator. If not, see <http://www.gnu.org/licenses/>.
 *
 */
public class TransparentOverlayFrame extends JFrame
{
  protected BufferedImage background;
  private JLabel jlabel = new JLabel();
  private int yOffset = 0;
  private JPanel contentPane;
  private XYLocator mainController;
  
  private boolean isMac;

  public TransparentOverlayFrame(XYLocator mainController, boolean isMac)
  {
    super();
    this.mainController = mainController;
    this.isMac = isMac;
    contentPane = (JPanel)this.getContentPane();
    contentPane.setDoubleBuffered(true);
    getContentPane().setLayout(new BorderLayout());
    
    // frame behavior
    setAlwaysOnTop(true);
    setUndecorated(true);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setSize(screenSize.width, screenSize.height - yOffset);
    addMouseListener();

    createImageOfDesktop();
    
    // create an instance of my component
    final XYLocationMouseCursor alsXYMouseLabel = new XYLocationMouseCursor();

    // add my component to the DRAG_LAYER of the layered pane
    JLayeredPane layeredPane = getRootPane().getLayeredPane();
    layeredPane.add(alsXYMouseLabel, JLayeredPane.DRAG_LAYER);
    alsXYMouseLabel.setBounds(0, 0, getWidth(), getHeight());

    // add a mouse motion listener, and update my mouse label with the x/y
    // coordinates as the user moves the mouse
    addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent me)
      {
        alsXYMouseLabel.x = me.getX();
        alsXYMouseLabel.y = me.getY();
        alsXYMouseLabel.repaint();
      }
    });
    
    // cheesy way to display an image 
    getContentPane().add("North", jlabel);
    setCursor(Cursor.CROSSHAIR_CURSOR);
  }

  void addMouseListener()
  {
    this.addMouseListener(new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        int button = e.getButton();
        if (button == MouseEvent.BUTTON1) {
          // get the color at this pixel
          Robot r = null;
          Color c = null;
          try {
            r = new Robot();
            c = r.getPixelColor(e.getX(),e.getY());
            mainController.doRecordCoordinatesAction(e.getX(), e.getY(), c);
          }
          catch (AWTException ex) {
            System.out.println(ex.getMessage());
            mainController.doRecordCoordinatesAction(e.getX(), e.getY());
          }
        }
      }
    });
  }

  public void paint(Graphics g)
  {
    super.paint(g);
    g.setColor(Color.ORANGE);
    Toolkit tk = Toolkit.getDefaultToolkit();
    Dimension dim = tk.getScreenSize();
    g.drawString("OVERLAY", dim.width/2-50, dim.height-40);
  }

  public void createImageOfDesktop()
  {
    try
    {
      Robot rbt = new Robot();
      Toolkit tk = Toolkit.getDefaultToolkit();
      Dimension dim = tk.getScreenSize();
      if (isMac)
      {
        yOffset = 22;
      }
      background = rbt.createScreenCapture(new Rectangle(0, yOffset, dim.width, dim.height-yOffset));
      jlabel.setIcon(new ImageIcon(background));
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

}











