package com.devdaily.xylocator;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.*;
import javax.swing.SwingUtilities;
import javax.swing.JFrame;
import com.apple.eawt.Application;

/**
 * XYLocator - A simple application to determine the X/Y coordinates of the cursor.
 * Also gets the RGB color at that point.
 * 
 * Copyright 2010, Alvin J. Alexander, devdaily.com.
 * 
 * This file is part of the devdaily.com XYLocator application.
 * 
 * XYLocator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * XYLocator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XYLocator. If not, see <http://www.gnu.org/licenses/>.
 *
 */
public class XYLocator
{
  private static final String APP_NAME = "XYLocator";
  private boolean isMac;
  
  TransparentOverlayFrame shield;
  XYLocator thisController;
  XYMainFrame mainFrame;

  public static void main(String[] args)
  {
    new XYLocator();
  }

  /**
   * If the app is not running on mac os x, die right away.
   */
  private void dieIfNotRunningOnMacOsX()
  {
    boolean mrjVersionExists = System.getProperty("mrj.version") != null;
    boolean osNameExists = System.getProperty("os.name").startsWith("Mac OS");
    
    if ( !mrjVersionExists || !osNameExists)
    {
      System.err.println("Not running on a Mac OS X system.");
      System.exit(1);
    }
    else
    {
      isMac = true;
    }
  }
  

  public XYLocator()
  {
    thisController = this;
    
    dieIfNotRunningOnMacOsX();
    
    configureForMacOSX();

    mainFrame = new XYMainFrame(this);
    mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    mainFrame.setLocationRelativeTo(null);
    
    // display the jframe
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        mainFrame.setVisible(true);
      }
    });
  }
  
  /**
   * Do everything we need to configure the app for Mac OS X systems.
   */
  private void configureForMacOSX()
  {
    // set some mac-specific properties; helps when i don't use ant to build the code
    System.setProperty("apple.awt.graphics.EnableQ2DX", "true");
    System.setProperty("apple.laf.useScreenMenuBar", "true");
    System.setProperty("com.apple.mrj.application.apple.menu.about.name", APP_NAME);

    // create an instance of the Mac Application class, so i can handle the 
    // mac quit event with the Mac ApplicationAdapter
    Application macApplication = Application.getApplication();
    MyApplicationAdapter macAdapter = new MyApplicationAdapter(this);
    macApplication.addApplicationListener(macAdapter);
  }
  
  
  public void doQuitAction()
  {
    System.exit(0);
  }
  
  public void doPreferencesAction()
  {
//    preferencesController.doHandlePreferencesEvent();
  }

  /**
   * this method should be called by the recorder.
   * we'll (a) put the information on the clipboard and (b) display it in the
   * main frame.
   */
  public void doRecordCoordinatesAction(int x, int y, Color c)
  {
    StringBuffer sb = new StringBuffer();
    sb.append("x, y: " + x + ", " + y + "\n");
    if (c != null)
    {
      sb.append("RGB: " + c.getRed() + ", " + c.getGreen() + ", " + c.getBlue() + "\n");
    }
    else
    {
      sb.append("could not obtain color information");
    }
    String s = sb.toString();
    writeToClipboard(s);
    shield.setVisible(false);
    shield.dispose();
    shield = null;
    mainFrame.setCoordinateInformation(s);
    mainFrame.setStatus("x/y and color placed on clipboard");
    mainFrame.setVisible(true);
  }

  public void doRecordCoordinatesAction(int x, int y)
  {
    doRecordCoordinatesAction(x,y,null);
  }

  public void doRecordCoordinatesAction()
  {
    System.exit(0);
  }

  public void doExitAction()
  {
    System.exit(0);
  }

  public static void writeToClipboard(String s)
  {
    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    Transferable transferable = new StringSelection(s);
    clipboard.setContents(transferable, null);
  }

  public void doGetXYAndColorAction()
  {
    // hide the main frame
    mainFrame.setVisible(false);

    // give the user a moment to click some other app and bring
    // it to the foreground
    try { Thread.sleep(1000);  }  catch (InterruptedException ie) {}

    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        shield = new TransparentOverlayFrame(thisController, isMac);
        shield.setVisible(true);
        shield.repaint();
      }
    });
  }
}

